#!/usr/bin/env python3

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Copyright 2013, by the California Institute of Technology. ALL RIGHTS RESERVED.
# United States Government Sponsorship acknowledged. Any commercial use must be
# negotiated with the Office of Technology Transfer at the California Institute of
# Technology.  This software is subject to U.S. export control laws and regulations
# and has been classified as EAR99.  By accepting this software, the user agrees to
# comply with all applicable U.S. export laws and regulations.  User has the
# responsibility to obtain export licenses, or other export authority as may be
# required before exporting such information to foreign countries or providing
# access to foreign persons.
#
# Author: Piyush Agram
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~




import datetime
import logging
import isceobj
from isceobj.Scene.Frame import Frame
from iscesys.Component.Component import Component

OUTPUT = Component.Parameter(
    'output',
    public_name='OUTPUT',
    default='',
    type=str,
    mandatory=False,
    intent='input',
    doc='Raw output file name.'
)
class Sensor(Component):
    """
    Base class for storing Sensor data
    """
    parameter_list = (
                      OUTPUT,                    
                     )
    logging_name =  None
    lookMap = {'RIGHT' : -1,
               'LEFT'  : 1}
    family = 'sensor'

    def __init__(self,family='',name=''):
        super(Sensor, self).__init__(family if family else  self.__class__.family, name=name)
        self.frame = Frame()
        self.frame.configure()

        self.logger = logging.getLogger(self.logging_name)

        self.frameList = []
       
        return None


    def getFrame(self):
        '''
        Return the frame object.
        '''
        return self.frame

    def parse(self):
        '''
        Dummy routine.
        '''
        raise NotImplementedError("In Sensor Base Class")


    def populateMetadata(self, **kwargs):
        """
        Create the appropriate metadata objects from our HDF5 file
        """
        self._populatePlatform(**kwargs)
        self._populateInstrument(**kwargs)
        self._populateFrame(**kwargs)
        self._populateOrbit(**kwargs)

    def _populatePlatform(self,**kwargs):
        '''
        Dummy routine to populate platform information.
        '''
        raise NotImplementedError("In Sensor Base Class")

    def _populateInstrument(self,**kwargs):
        """
        Dummy routine to populate instrument information.
        """
        raise NotImplementedError("In Sensor Base Class")

    def _populateFrame(self,**kwargs):
        """
        Dummy routine to populate frame object.
        """
        raise NotImplementedError("In Sensor Base Class")

    def _populateOrbit(self,**kwargs):
        """
        Dummy routine to populate orbit information.
        """
        raise NotImplementedError("In Sensor Base Class")

    def extractImage(self):
        """
        Dummy routine to extract image.
        """
        raise NotImplementedError("In Sensor Base Class")

    def extractDoppler(self):
        """
        Dummy routine to extract doppler centroid information.
        """
        raise NotImplementedError("In Sensor Base Class")
